---
title: Installation with Helm
description: Install Istio with the included Helm chart.
weight: 30
keywords: [kubernetes,helm]
aliases:
    - /docs/setup/kubernetes/helm.html
    - /docs/tasks/integrating-services-into-istio.html
page_icon: /img/helm.svg
---

Quick start instructions for the setup and configuration of Istio using Helm.
This is the recommended install method for installing Istio to your
production environment as it offers rich customization to the Istio control
plane and the sidecars for the Istio data plane.

## Prerequisites

1. [Download the Istio release](/docs/setup/kubernetes/download-release/).

1. [Kubernetes platform setup](/docs/setup/kubernetes/platform-setup/):
  * [Minikube](/docs/setup/kubernetes/platform-setup/minikube/)
  * [Google Container Engine (GKE)](/docs/setup/kubernetes/platform-setup/gke/)
  * [IBM Cloud Kubernetes Service (IKS)](/docs/setup/kubernetes/platform-setup/ibm/)
  * [OpenShift Origin](/docs/setup/kubernetes/platform-setup/openshift/)
  * [Amazon Web Services (AWS) with Kops](/docs/setup/kubernetes/platform-setup/aws/)
  * [Azure](/docs/setup/kubernetes/platform-setup/azure/)

1. [Install the Helm client](https://docs.helm.sh/using_helm/#installing-helm).

1. Istio by default uses LoadBalancer service object types.  Some platforms do not support LoadBalancer
   service objects.  For platforms lacking LoadBalancer support, install Istio with NodePort support
   instead with the flags `--set gateways.istio-ingressgateway.type=NodePort --set gateways.istio-egressgateway.type=NodePort`
   appended to the end of the helm operation.

## Option 1: Install with Helm via `helm template`

1. Render Istio's core components to a Kubernetes manifest called `istio.yaml`:

    {{< text bash >}}
    $ helm template install/kubernetes/helm/istio --name istio --namespace istio-system > $HOME/istio.yaml
    {{< /text >}}

1. Install the components via the manifest:

    {{< text bash >}}
    $ kubectl create namespace istio-system
    $ kubectl create -f $HOME/istio.yaml
    {{< /text >}}

## Option 2: Install with Helm and Tiller via `helm install`

This option allows Helm and
[Tiller](https://github.com/kubernetes/helm/blob/master/docs/architecture.md#components)
to manage the lifecycle of Istio.

1. If a service account has not already been installed for Tiller, install one:

    {{< text bash >}}
    $ kubectl create -f install/kubernetes/helm/helm-service-account.yaml
    {{< /text >}}

1. Install Tiller on your cluster with the service account:

    {{< text bash >}}
    $ helm init --service-account tiller
    {{< /text >}}

1. Install Istio:

    {{< text bash >}}
    $ helm install install/kubernetes/helm/istio --name istio --namespace istio-system
    {{< /text >}}

## Customization Example: Traffic Management Minimal Set

Istio has a rich feature set, but you may only want to use a subset of these. For instance, you might be only interested in installing the minimum necessary to support traffic management functionality.

This example shows how to install the minimal set of components necessary to use [traffic management](/docs/tasks/traffic-management/) features.

Execute the following command to install the Pilot and Citadel:

{{< text bash >}}
$ helm install install/kubernetes/helm/istio --name istio --namespace istio-system \
  --set ingress.enabled=false \
  --set gateways.istio-ingressgateway.enabled=false \
  --set gateways.istio-egressgateway.enabled=false \
  --set galley.enabled=false \
  --set sidecarInjectorWebhook.enabled=false \
  --set mixer.enabled=false \
  --set prometheus.enabled=false \
  --set global.proxy.envoyStatsd.enabled=false
{{< /text >}}

Ensure the `istio-pilot-*` and `istio-citadel-*` Kubernetes pods are deployed and their containers are up and running:

{{< text bash >}}
$ kubectl get pods -n istio-system
NAME                                     READY     STATUS    RESTARTS   AGE
istio-citadel-b48446f79-wd4tk            1/1       Running   0          1m
istio-pilot-58c65f74bc-2f5xn             2/2       Running   0          1m
{{< /text >}}

With this minimal set you can install your own application and [configure request routing](/docs/tasks/traffic-management/request-routing/) for instance. You will need to [manually inject the sidecar](/docs/setup/kubernetes/sidecar-injection/#manual-sidecar-injection).

[Installation Options](/docs/reference/config/installation-options/) has the full list of options allowing you to tailor the Istio installation to your needs. Before you override the default value with `--set` in `helm install`, please check the configurations for the option in `install/kubernetes/helm/istio/values.yaml` and uncomment the commented context if needed.

## Uninstall

* For option 1, uninstall using `kubectl`:

    {{< text bash >}}
    $ kubectl delete -f $HOME/istio.yaml
    {{< /text >}}

* For option 2, uninstall using Helm:

    {{< text bash >}}
    $ helm delete --purge istio
    {{< /text >}}

    If your helm version is less than 2.9.0, then you need to manually cleanup extra job resource before redeploy new version of Istio chart:

    {{< text bash >}}
    $ kubectl -n istio-system delete job --all
    {{< /text >}}
